% Kepler's perturbed 2 body problem.

% Copyright 2005-2005 Patrick Meury
% SAM - Seminar for Applied Mathematics
% ETH-Zentrum
% CH-8092 Zurich, Switzerland

  h = 0.01;                               % Steps size
  nSteps = 10000;                         % Time steps
  tol = 1e-8;                             % Error tolerance
  maxit = 20;                             % Maximum iterations
  e = 0.6;                                % Eccentricity
  mu = 0.005;                             % Relativistic parameter
  y0 = [1-e; 0; 0; sqrt(1+e)/sqrt(1-e)];  % Initial value
  f_handle = @f_pkepler;                  % Right hand side
  g_handle = @g_pkepler;                  % Constraints
  
  % Compute total energy and angular momentum
  
  g = g_handle(y0,mu,0,0);
  H0 = g(1);
  L0 = g(2);
  
  % Run Euler schemes
  
  y1 = expl_Eul(y0,h,nSteps,f_handle,mu);
  y2 = expl_Eul_tsp(y0,h,nSteps,tol,maxit,f_handle,g_handle,mu,H0);
  y3 = expl_Eul_tsp(y0,h,nSteps,tol,maxit,f_handle,g_handle,mu,H0,L0);
  
  % Compute values of constraints
  
  g1 = g_handle(y1,mu,H0,L0);
  g2 = g_handle(y2,mu,H0,L0);
  g3 = g_handle(y3,mu,H0,L0);
  
  % Generate figures
  
  figure('Name','Perturbed Kepler problem');
  plot(y1(1,:),y1(2,:),'r-');
  legend('expl. Euler','Location','SouthEast');
  title('{\bf Perturbed Kepler problem}');
  xlabel('{\bf q_1}');
  ylabel('{\bf q_2}');
  set(gca,'XLim',[-2 2],'YLim',[-2 2],'DataAspectRatio',[1 1 1]);
 
  figure('Name','Perturbed Kepler problem');
  plot(y2(1,:),y2(2,:),'b-');
  legend('expl. Euler tsp','Location','SouthEast');
  title('{\bf Perturbed Kepler problem}');
  xlabel('{\bf q_1}');
  ylabel('{\bf q_2}');
  set(gca,'XLim',[-2 2],'YLim',[-2 2],'DataAspectRatio',[1 1 1]);
  
  figure('Name','Perturbed Kepler problem');
  plot(y3(1,:),y3(2,:),'g-');
  legend('expl. Euler tsp','Location','SouthEast');
  title('{\bf Perturbed Kepler problem}');
  xlabel('{\bf q_1}');
  ylabel('{\bf q_2}');
  set(gca,'XLim',[-2 2],'YLim',[-2 2],'DataAspectRatio',[1 1 1]);
  
  figure('Name','Constraints');
  plot(0:1:nSteps,sqrt(sum(g1.^2,1)),'r.', ...
       0:1:nSteps,sqrt(sum(g2.^2,1)),'b.', ...
       0:1:nSteps,sqrt(sum(g3.^2,1)),'g.', ...
       'MarkerSize',5);
  legend('expl. Euler','expl. Euler tsp','expl. Euler tsp', ...
         'Location','SouthEast');
  title('{\bf l^2-norm of constraint}');
  xlabel('{\bf Step number}');
  ylabel('{\bf Constraint [log]}');
  set(gca,'YScale','log');
  
  % Clear memory
  
  clear all;
  