function y = expl_Eul_tsp(y0,h,nSteps,tol,maxit,f_handle,g_handle,varargin)
% EXPL_EUL_TSP Explicit Euler scheme with tangent space parametrization.
%
%   Y = EXPL_EUL_TSP(Y0,H,NSTEPS,TOL,MAXIT,F_HANDLE,G_HANDLE) computes the
%   solution to the autonomous ordinary differential equation
%
%     d/dt y = f(y) in (0,T),
%       y(0) = y0
%
%   on the manifold given by the constraint G_HANDLE using NSTEPS time
%   steps with step size H and initial data Y0 on the manifold. To ensure
%   that the solution stays on the manifold the tangent space paramet-
%   rization is used together with the Newton method with tolerance TOL and
%   maximum number of iterations MAXIT.
%
%   Example:
%
%   y = expl_Eul_tsp(y0,h,nSteps,@tol,maxit,@f_mp,@g_mp);

%   Copyright 2005-2005 Patrick Meury
%   SAM - Seminar for Applied Mathematics
%   ETH-Zentrum
%   CH-8092 Zurich, Switzerland

  % Initialize constants
  
  dim = size(y0,1);  % Number of ODEs 
  
  % Preallocate memory
  
  y = zeros(dim,nSteps+1);
 
  % Copy initial value
  
  y(:,1) = y0;
  
  % Start explicit Euler scheme
  
  for i = 1:nSteps
  
    % Construct orthogonal basis for the tangent space, null space of the
    % jacobian of the constraint g, by using a single value decomposition

    a = y(:,i);
    [g_a,gp_a] = g_handle(a,varargin{:});    
    [m,n] = size(gp_a);
    [U,S,V] = svd(gp_a,0);
    rk = sum(diag(S) > tol);
    Q = V(:,rk+1:n);
    
    % Do 1 explicit Euler step in the parameter domain
    
    z = h*transpose(Q)*f_handle(a,varargin{:});
    
    % Project solution onto manifold using tangent space parametrization 
    
    u_new = zeros(m,1);
    delta = tol+1;
    iter = 0;
    while(delta > tol & iter < maxit)
      u_old = u_new;
      [g,gp] = g_handle(a+Q*z+transpose(gp_a)*u_new,varargin{:});
      u_new = u_new - (gp*transpose(gp_a))\g;      
      delta = max(norm(g),norm(u_old-u_new));
      iter = iter+1;
    end
    y(:,i+1) = a + Q*z + transpose(gp_a)*u_new;
    
  end
  
return