function y = expl_RKM(y0,h,nSteps,B,C,f_handle,varargin)
% EXPL_RKM Explicit Runge-Kutta method.
%
%   Y = EXPL_RKM(Y0,H,NSTEPS,B,C,F_HANDLE) computes the solution to the auto-
%   nomous ordinary differential equation
%
%     d/dt y = f(y) in (0,T),
%       y(0) = y0
%
%   using the NSTEPS time steps with step size H and initial data Y0.
%
%   Example:
%
%   y = expl_RKM(y0,h,nSteps,B,C,@f_mp);

%   Copyright 2005-2005 Patrick Meury
%   SAM - Seminar for Applied Mathematics
%   ETH-Zentrum
%   CH-8092 Zurich, Switzerland

  % Initialize constants
  
  dim = size(y0,1);  % Number of ODEs 
  s = size(B,1)+1;   % Number of stages for Runge-Kutta scheme
  
  % Preallocate memory
  
  y = zeros(dim,nSteps+1);
  k = zeros(dim,s);
  
  % Copy initial value
  
  y(:,1) = y0;
  
  % Start explicit Runge-Kutta method
  
  for i = 1:nSteps  
    k(:,1) = f_handle(y(:,i),varargin{:});
    y(:,i+1) = y(:,i) + h*C(1)*k(:,1);
    for j1 = 2:s
      z = y(:,i);
      for j2 = 1:j1-1
        z = z + h*B(j1-1,j2)*k(:,j2);    
      end
      k(:,j1+1) = f_handle(z,varargin{:});
      y(:,i+1) = y(:,i+1) + h*C(j1)*k(:,j1);
    end
  end

return