function y = impl_mdpt(y0,h,nSteps,tol,maxit,f_handle,varargin)
% IMPL_MDPT Implicit midpoint scheme.
%
%   Y = IMPL_MDPT(Y0,H,NSTEPS,TOL,MAXIT,F_HANDLE) computes the solution to
%   the autonomous ordinary differential equation
%
%     d/dt y = f(y) in (0,T),
%       y(0) = y0
%
%   using the NSTEPS time steps with step size H and initial data Y0.
%   Banach iterations are used with tolerance TOL and maximum number of
%   iterations MAXIT to solve the fix point equation.
%
%   Example:
%
%   y = impl_mdpt(y0,h,nSteps,tol,maxit,@f_mp);

%   Copyright 2005-2005 Patrick Meury
%   SAM - Seminar for Applied Mathematics
%   ETH-Zentrum
%   CH-8092 Zurich, Switzerland

  % Initialize constants

  dim = size(y0,1);  % Number of ODEs

  % Preallocate memory
  
  y = zeros(dim,nSteps+1);

  % Copy initial value
  
  y(:,1) = y0;
  
  % Start implicit Euler scheme
  
  for i = 1:nSteps
      
    % Do fix point iterations until stopping criteria are met  
      
    z_new = y(:,i) + h*f_handle(y(:,i),varargin{:});
    delta = tol+1;
    iter = 0;
    while(delta > tol & iter < maxit)
      z_old = z_new;
      z_new = y(:,i) + h*f_handle((y(:,i)+z_new)/2,varargin{:});
      delta = norm(z_new-z_old);
      iter = iter+1;
    end    
        
    % Update solution to ODE system
    
    y(:,i+1) = z_new;
    
  end
    
return